#!/usr/bin/env python3

import json
import logging

import click

from habu.lib.iface import get_ifaces


@click.command()
@click.option('-j', 'json_output', is_flag=True, default=False, help='Output in JSON format')
def cmd_interfaces(json_output):
    """
    Show the network interfaces available on the system.

    Example:

    \b
    # habu.interfaces
    #  NAME                            MAC                INET             INET6
    0  eth0                            80:fa:5b:4b:f9:18  None             None
    1  lo                              00:00:00:00:00:00  127.0.0.1        ::1
    2  wlan0                           f4:96:34:e5:ae:1b  192.168.0.6      None
    3  vboxnet0                        0a:00:27:00:00:00  192.168.56.1     fe80::800:27ff:fe00:0
    """

    interfaces = get_ifaces()

    if json_output:
        print(json.dumps(interfaces, indent=4))
        return True

    print('{:<3}{:<32}{:<19}{:<17}{}'.format('#', 'NAME', 'MAC', 'INET', 'INET6'))
    for interface in interfaces.values():
        print('{index:<3}{name:<32}{mac:<19}{inet:<17}{inet6}'.format(
            index=str(interface['index']),
            name=str(interface['name']),
            mac=str(interface['mac']),
            inet=str(interface['inet']),
            inet6=str(interface['inet6'])
        ))


if __name__ == '__main__':
    cmd_interfaces()



import sys

# Modified version of https://github.com/barneygale/elevate/

def elevate(show_console=True):
    """
    Re-launch the current process with root/admin privileges

    When run as root, this function does nothing.

    When not run as root, this function replaces the current process (Linux,
    macOS) or creates a child process, waits, and exits (Windows).

    :param show_console: (Windows only) if True, show a new console for the
        child process. Ignored on Linux / macOS.
    :param graphical: (Linux / macOS only) if True, attempt to use graphical
        programs (gksudo, etc). Ignored on Windows.
    """
    if sys.platform.startswith("win"):
        from habu.lib.elevate.windows import elevate
    else:
        from habu.lib.elevate.posix import elevate

    elevate(show_console)




import errno
import os
import sys
try:
    from shlex import quote
except ImportError:
    from pipes import quote


def quote_shell(args):
    return " ".join(quote(arg) for arg in args)


def quote_applescript(string):
    charmap = {
        "\n": "\\n",
        "\r": "\\r",
        "\t": "\\t",
        "\"": "\\\"",
        "\\": "\\\\",
    }
    return '"%s"' % "".join(charmap.get(char, char) for char in string)


def elevate(show_console=True):
    if os.getuid() == 0:
        return

    args = [sys.executable] + sys.argv

    command = ["sudo"] + args

    try:
        os.execlp(command[0], *args)
    except OSError as e:
        if e.errno != errno.ENOENT or args[0] == "sudo":
            raise



import sys
from scapy.all import conf, get_if_hwaddr


if sys.platform.startswith('win'):
    from scapy.all import get_windows_if_list
    interfaces = get_windows_if_list()
    windows = True
else:
    from scapy.all import get_if_list
    interfaces = get_if_list()
    windows = False


def get_ifaces():
    """Get a list of network interfces.
    Returns a list like this:

    {'eth0': {'index': 0,
              'inet': None,
              'inet6': None,
              'mac': '80:fa:5b:4b:f9:18',
              'name': 'eth0'},
     'lo': {'index': 1,
            'inet': '127.0.0.1',
            'inet6': '::1',
            'mac': '00:00:00:00:00:00',
            'name': 'lo'},
     'vboxnet0': {'index': 3,
                  'inet': '192.168.56.1',
                  'inet6': 'fe80::800:27ff:fe00:0',
                  'mac': '0a:00:27:00:00:00',
                  'name': 'vboxnet0'},
     'wlan0': {'index': 2,
               'inet': '192.168.0.6',
               'inet6': None,
               'mac': 'f4:96:34:e5:ae:1b',
               'name': 'wlan0'}}
    """

    result = {}
    index = 0

    for i in interfaces:
        # Unix return strings and Windows dictionaries
        if isinstance(i, str):
            name = i
        elif isinstance(i, dict):
            name = i['name']
        else:
            print('Unexpected result', file=sys.stderr)
            return {}

        result[name] = { 'name' : name, 'index' : index }
        if windows:
            result[name]['mac'] = i['mac']
        else:
            result[name]['mac'] = get_if_hwaddr(name)

        if not result[name]['mac']:
            result[name]['mac'] = None

        result[name]['inet'] = None
        for route in conf.route.routes:
            if getattr(route[3], 'name', route[3]) == name:
                result[name]['inet'] = route[4]
                break

        result[name]['inet6'] = None
        for route in conf.route6.routes:
            if getattr(route[3], 'name', route[3]) == name:
                result[name]['inet6'] = route[4][0]
                break

        index += 1

    return result


def search_iface(term):
    """Get a valid interface based on a search term, that can be:
    The interface name
    The interface index - This is NOT the Windows Interface Index
    The interface inet address
    The interface inet6 address
    The interface mac address"""

    if not term:
        return None

    for iface in get_ifaces().values():
        if term in iface.values():
            return iface

        if str(term) == str(iface['index']):
            return iface

    return None


if __name__ == '__main__':
    from pprint import pprint
    pprint(get_ifaces())
